/**
 *  PaintDomeServer2018B is starting point for final
 *  Assignment 4 in CSC220 Fall 2018, following from
 *  Assignment 3. Both PaintDomeServer2018B and instructor-supplied
 *  PaintDomeAndroid2018B use a different, enhanced OSC/UDP
 *  application-level communication protocol that differs
 *  from Assignment 3.
 */
 
import oscP5.*;
import netP5.*;
import java.util.* ;
import java.util.concurrent.ConcurrentLinkedQueue ;

// Use -1 for midiDeviceIndex until you know which to use.
final int midiDeviceIndex = 0 ;  // setup() checks for number of devices. Use one for output.

// CARRIED OVER FROM sketch PaintDome2018, csc120, spring 2018:
/* Author: Dr. Parson
/* Original Creation Date: 4/15/2018
/* Enhanced for fall 2018 CSC220 in November 2018
/* Student paintbrush due date: 12/8/2018
/* Sketch name: PaintDomeServer2018B
/* USER INTERACTION:
/* +STUDENTID\n to add a student PaintBrush.
/* -STUDENTID\n to to remove a student PaintBrush.
/* 'C' to clear display (erase the paint)
/* 'R' to reset to start of program, with no PaintBrushs
/* 'F' to freeze the display
/* 'M' toggles moveMode for paintbrushes to stop them temporarily
/* '~' calls configure() on all visible PaintBrushes to move them somewhere else.
/* 'Z' calls zap() on all visible PaintBrushes.
/* 'D' toggles the global dilate filter switch.
/* 'E' toggles the global erode filter switch.
/* 'P' toggles the global posterize 2 filter switch.
/*  Per-PaintBrush modifications come via OSC
/*    Hit LEFT, RIGHT, UP, or DOWN KEY to change a property that you define.
/* STUDENT WORK GOES INTO A CLASS BELOW NAMED AFTER YOUR KUTZTOWN EMAIL ID.
/* See interface PaintBrush, class parson, and your individual class.
/* OSC PROTOCOL: all datagrams from client start out with
/*       /paintdome18B/command IPCLIENTADDRSTRING UDPCLIENTPORTINT brushname floatvalue
/*            where the brushname is ignored by global commands Zap, Dilate, Erode, Poster,
/*            and floatvalue is ignored by commands that don't need it.

/*    a) Setting IP of PaintDomeServer2018B process, sending
/*       initial signup, and receiving reply.
/*       CLIENT: SEND DATAGRAM: /paintdome18B/client IPCLIENTADDRSTRING UDPCLIENTPORTINT ignorestring ignorefloat
/*       SERVER: RECV DATAGRAM: /paintdome18B/server strvalBrushName isPaintbrushVisibleINT
/*          With the goal of not over-running the max size of a reply
/*          (a.k.a. the Maximum Transmission Unit or MTU), the server sends
/*          back a series of these replies.
/*    b) Sending commands to current paintbrush from this set,
/*       with non-index parameters sent as float values.
/*       The networked OSC/IP commands are OSC addres pattern strings
/*          given below, which must also be used in PaintDomeServer2018.
/*          and each client datagram has paramas CLIIP CLIPORT (client IP & PORT)
/*          add namedPaintbrush (make it visible) '+'
/*            /paintdome18B/addbrush CLIIP CLIPORT brushname floatvalIgnore
/*          remove namedPaintbrush (make it invisible) '-'
/*            /paintdome18B/rembrush CLIIP CLIPORT brushname floatvalIgnore
/*          shuffle namedPaintbrush (calls its shuffle function to randomize location)
/*            /paintdome18B/shuffle CLIIP CLIPORT brushname floatvalIgnore '~'
/*          set locationX
/*            /paintdome18B/setX CLIIP CLIPORT brushname floatval
/*          set locationY
/*            /paintdome18B/setY CLIIP CLIPORT brushname floatval
/*          set speedX (+ is right, - is left)
/*            /paintdome18B/speedX CLIIP CLIPORT brushname floatval
/*          set speedY (+ is down, - is up)
/*            /paintdome18B/speedY CLIIP CLIPORT brushname floatval
/*          set scaleX
/*            /paintdome18B/scaleX CLIIP CLIPORT brushname floatval
/*          set speedScaleX
/*            /paintdome18B/scaleXS CLIIP CLIPORT brushname floatval
/*          set scaleXLowerBound (may be negative)
/*            /paintdome18B/scaleXLB CLIIP CLIPORT brushname floatval
/*          set scaleXUpperBound (may be negative)
/*            /paintdome18B/scaleXUB CLIIP CLIPORT brushname floatval
/*          set scaleY
/*            /paintdome18B/scaleY CLIIP CLIPORT brushname floatval
/*          set speedScaleY
/*            /paintdome18B/scaleYS CLIIP CLIPORT brushname floatval
/*          set scaleYLowerBound (may be negative)
/*            /paintdome18B/scaleYLB CLIIP CLIPORT brushname floatval
/*          set scaleYUpperBound (may be negative)
/*            /paintdome18B/scaleYUB CLIIP CLIPORT brushname floatval
/*          set rotate in degrees (may be negative)
/*            /paintdome18B/rotate CLIIP CLIPORT brushname floatval
/*          set speedRotate
/*            /paintdome18B/speedRot CLIIP CLIPORT brushname floatval
/*          set rotateBound (symmetric +/-, use NOROTATEBOUND for no bound)
/*            /paintdome18B/rotateB CLIIP CLIPORT brushname floatval
/*          set shearX in degrees (may be negative)
/*            /paintdome18B/shearX CLIIP CLIPORT brushname floatval
/*          set speedShearX
/*            /paintdome18B/shearXS CLIIP CLIPORT brushname floatval
/*          set shearXBound (symmetric +/-, use NOROTATEBOUND for no bound)
/*            /paintdome18B/shearXB CLIIP CLIPORT brushname floatval
/*          set shearY in degrees (may be negative)
/*            /paintdome18B/shearY CLIIP CLIPORT brushname floatval
/*          set speedShearY
/*            /paintdome18B/shearYS CLIIP CLIPORT brushname floatval
/*          set shearYBound (symmetric +/-, use NOROTATEBOUND for no bound)
/*            /paintdome18B/shearYB CLIIP CLIPORT brushname floatval
/*          reset paintbrush to its default values for all of above parameters.
/*            /paintdome18B/reset CLIIP CLIPORT brushname floatval(ignored)
/*          GLOBAL COMMANDS AFFECT ENTIRE CANVAS:
/*          'Z' calls zap() on all visible PaintBrushes.
/*            /paintdome18B/Zap CLIIP CLIPORT brushname(ignored) floatval(ignored)
/*          'D' toggles the global dilate filter switch.
/*             /paintdome18B/Dilate CLIIP CLIPORT brushname(ignored) floatval(ignored)
/*          'E' toggles the global erode filter switch.
/*             /paintdome18B/Erode CLIIP CLIPORT brushname(ignored) floatval(ignored)
/*          'P' sets the posterize level.
/*             /paintdome18B/Poster CLIIP CLIPORT brushname(ignored) floatval-for-posterization
/*          'C' clears the display.
/*             /paintdome18B/Clear CLIIP CLIPORT brushname(ignored) floatval(ignored)
/*          'F' toggles freeze of the display.
/*             /paintdome18B/Freeze CLIIP CLIPORT brushname(ignored) floatval(ignored)
/*          'R' resets the server.
/*             /paintdome18B/Reset CLIIP CLIPORT brushname(ignored) floatval(ignored)
/*          'M' toggles paintbrush movement.
/*             /paintdome18B/Move CLIIP CLIPORT brushname(ignored) floatval(ignored)
 *********************************************************/
 
OscP5 oscP5;
NetAddress myRemoteLocation;

// TreeMap sorts brush names in alphabetical order.
TreeMap<String, PaintBrush> allshapes = new TreeMap<String, PaintBrush>();
TreeMap<String, PaintBrush> showshapes = new TreeMap<String, PaintBrush>();
int paddedNameWidth = 9 ; // student IDs are 8 chars, add +1
PImage clippingCircle ;  // This clips the display to a circle in the planetarium dome.
color backgroundColor ;
boolean eraseMode = true ; // 'e' toggles this, true runs background().
boolean moveMode = true ; // toggle to disable paintbrush moves
boolean freezeMode = false ; // toggle frozen/unfrozen display
boolean dilateMode = false, erodeMode = false ;
int posterizeLevel = 0 ;
int diameter = 0, radius = 0 ; // diameter and radius of central clipping circle

String [] commandStrings = {
    "addbrush", "rembrush", "shuffle", "setX", "setY",
    "speedX", "speedY", "scaleX", "scaleXS", "scaleXLB",
    "scaleXUB", "scaleY", "scaleYS", "scaleYLB", "scaleYUB",
    "rotate", "speedRot", "rotateB", "shearX", "shearXS",
    "shearXB", "shearY", "shearYS", "shearYB", "reset",
    "Zap", "Dilate", "Erode", "Posterize", "Clear", "Move",
    "Freeze", "Reset"
};
HashSet<String> commandNames = new HashSet<String>(); // copy of commandStrings
final float noRotateB = Float.MAX_VALUE ; // in this case do not reverse rotation at bound

// oscEvent runs in a different thread, so send its data through a thread-safe queue to draw()
class IncomingOSC {
  final String clientIP ;
  final int clientPort ;
  final String command ;
  final String paintbrush ;
  final float fparameter ;
  IncomingOSC(
      String clientIP, int clientPort,
      String command, String paintbrush, float fparameter) {
    this.clientIP = clientIP ;
    this.clientPort = clientPort ;
    this.command = command ;
    this.paintbrush = paintbrush ;
    this.fparameter = fparameter ;
  }
};
// This Queue brings client OSC messages to the Processing GUI thread:
final ConcurrentLinkedQueue<IncomingOSC> IncomingOSCqueue = new ConcurrentLinkedQueue<IncomingOSC>();
// This set used to keep a set of client IP:port pairs:
final Set<IncomingOSC> ClientIPset = Collections.synchronizedSet(new HashSet<IncomingOSC>());

// MIDI STUFF
import javax.sound.midi.* ;  // Get everything in the Java MIDI (Musical Instrument Digital Interface) package.
// See http://midi.teragonaudio.com/tech/midispec.htm for CONTROLLER codes
final int CTRLNUM = 0 ; // CTRLNUM+0 through CTRLNUM+5 used for Max Surround Panner
final int CONTROLLER = 0xb0 ; // MIDI Controller opcode
// MIDI VARIABLES, see http://faculty.kutztown.edu/parson/spring2017/MidiKBSpring2016Parson.txt
// NOTE: A final variable is in fact a constant that cannot be changed.
MidiDevice.Info[] midiDeviceInfo = null ;
// See javax.sound.midi.MidiSystem and javax.sound.midi.MidiDevice
MidiDevice device = null ;
// See javax.sound.midi.MidiSystem and javax.sound.midi.MidiDevice
Receiver receiver = null ;
// javax.sound.midi.Receiver receives your OUTPUT MIDI messages (counterintuitive?)
// SEE https://www.midi.org/specifications/item/gm-level-1-sound-set but start at 0, not 1
final int NUMINSTRUMENTS = 16 ; // 16 MIDI channels before we have to recycle them.
int [] majorscale = {0, 2, 4, 5, 7, 9, 11};
int [] minorscale = {0, 2, 3, 5, 7, 8, 10};
final int octaves = 10 ; // 12 notes in a chromatic scale X 10 = 120; MIDI actually has 0..127 = 128

void setup() {
  // fullScreen(P2D);
  size(1900, 1080, P2D);
  frameRate(60);  // avoid Apple bug added 11/28/2021
  // size(700,700,P2D);
  // FROM SPRING 2018:
  textMode(SHAPE) ;
  // Hue goes 0..960 (see Tools -> Color Selector).
  // Saturation, Brightness, and Alpha limits are 0..100%.
  // Do not use RGB colorMode values in this program.
  colorMode(HSB, 360, 100, 100, 100);
  backgroundColor = color(0, 0, 0); // brightness of 0% is always black
  background(backgroundColor);
  rectMode(CENTER);
  ellipseMode(CENTER);
  imageMode(CENTER);
  shapeMode(CENTER);
  clippingCircle = loadImage("CenteredClippingCircle.png");
  diameter = (height < width) ? height : width ; // sets diameter to the smaller
  radius = diameter / 2 ;
  setupPaintBrushes();
    // MIDI:
  // 1. FIND OUT WHAT MIDI DEVICES ARE AVAILABLE FOR VARIABLE midiDeviceIndex.
  int i = 0 ;
  midiDeviceInfo = MidiSystem.getMidiDeviceInfo();
  for (i = 0; i < midiDeviceInfo.length; i++) {
    println("MIDI DEVICE NUMBER " + i + " Name: " + midiDeviceInfo[i].getName()
      + ", Vendor: " + midiDeviceInfo[i].getVendor()
      + ", Description: " + midiDeviceInfo[i].getDescription());
  }
  // 2. OPEN ONE OF THE MIDI DEVICES UP FOR OUTPUT.
  try {
    Exception rx = null ;
    if (midiDeviceIndex >= 0) {
      device = MidiSystem.getMidiDevice(midiDeviceInfo[midiDeviceIndex]);
      device.open();  // Make sure to close it before this sketch terminates!!!
      // There should be a way to schedule a method when Processing closes this
      // sketch, so we can close the device there, but it is not documented for Processing 3.
      try {
        receiver = device.getReceiver();
      } catch (Exception rxx) {
        rx = rxx ;
      }
    }
    if (receiver == null) {
      println("ERROR, MIDI midiDeviceIndex " + midiDeviceIndex + " does not have a receiver "
        + ((rx == null) ? "" : rx.getMessage()));
    }
    // NOTE: Either of the above method calls can throw MidiUnavailableException
    // if there is no available device or if it does not have a Receiver to
    // which we can send messages. The catch clause intercepts those error messages.
    // See https://www.midi.org/specifications/item/gm-level-1-sound-set, use voice variable
    /*
    ShortMessage noteMessage = new ShortMessage() ;
     noteMessage.setMessage(ShortMessage.PROGRAM_CHANGE, 0, voice, 0); // to channel 0
     receiver.send(noteMessage, -1L);  // send it now
     */
  } 
  catch (MidiUnavailableException mx) {
    println("MIDI UNAVAILABLE " + mx.getMessage()); // Error messages go here.
    device = null ;
    receiver = null ; // Do not try to use them.
    //exit();
  } catch (ArrayIndexOutOfBoundsException aix) {
    println("ArrayIndexOutOfBoundsException, Invalid midiDeviceIndex " + midiDeviceIndex + ", " + aix.getMessage());
  }
  /* start oscP5, listening for incoming messages at port 12001 */
  oscP5 = new OscP5(this,12001);
  
  /* myRemoteLocation is a NetAddress. a NetAddress takes 2 parameters,
   * an ip address and a port number. myRemoteLocation is used as parameter in
   * oscP5.send() when sending osc packets to another computer, device, 
   * application. usage see below. for testing purposes the listening port
   * and the port of the remote location address are the same, hence you will
   * send messages back to this sketch.
   */
  myRemoteLocation = new NetAddress("127.0.0.1",12001);
  for (String s : commandStrings) {
    commandNames.add(s.trim());
  }
  
  // println(" DEBUG setup() runs in thread: " + Thread.currentThread().toString());
}

void draw() {
  processOSCevents();
  pushMatrix(); // draw() plots relative to center of dome.
  translate(width/2, height/2); // Plot relative to center of the dome.
  if (freezeMode) {
    popMatrix();
    // println("DEBUG draw() runs in thread: " + Thread.currentThread().toString());
    return;
  }
  colorMode(HSB, 360, 100, 100, 100);
  rectMode(CENTER);
  ellipseMode(CENTER);
  imageMode(CENTER);
  for (PaintBrush nextShape : showshapes.values()) {
    // This kind of for loop loops over all values in showshapes.
    // println("DEBUG: showing brush " + nextShape.getName());
    colorMode(HSB, 360, 100, 100, 100); // in case a previous call to
    rectMode(CENTER);                   // display() changed these:
    ellipseMode(CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    textAlign(CENTER, CENTER);
    noTint();
    noFill();
    noStroke();
    strokeWeight(1);
    nextShape.display();
    if (moveMode) {
      nextShape.move();
    }
  }
  // Each of R,G,B gets 8 possible values instead of 256 in this call.
  if (posterizeLevel > 1) {
    filter(POSTERIZE,posterizeLevel);
  }
  // Dilate what was already painted.
  if (dilateMode) {
    filter(DILATE);
  }
  // Erode what was already painted. Only 1 of dilateMode and erodeMode
  // is turned on in keyPressed() at a time.
  if (erodeMode) {
    filter(ERODE);
  }
  tint(0, 0, 0, 100); // undo any earlier tint, HSB mode
  fill(0, 0, 0, 100); // likewise fill and stroke
  stroke(0, 0, 0, 100);
  image(clippingCircle, 0, 0, diameter, diameter);
  strokeWeight(1);
  popMatrix(); // draw() plots relative to center of dome.
  rectMode(CORNER); // just for plotting the clipping boxes
  rect(0,0, (width-diameter)/2, height);
  rect((width-diameter)/2+diameter,0, (width-diameter)/2, height);
  rectMode(CENTER);
  clip(width/2, height/2, diameter, diameter); // uses imageMode to clip boundaries.
} //<>// //<>//
// end draw() function


/* incoming osc message are forwarded to the oscEvent method. */
// oscEvent runs in an OSC thread, so send command via a thread-safe queue
void oscEvent(OscMessage theOscMessage) {
  String addrstr = theOscMessage.addrPattern();
  Object [] args = theOscMessage.arguments();
  if (addrstr != null && addrstr.startsWith("/paintdome18B/")) {
    String cmd = addrstr.substring(addrstr.lastIndexOf('/')+1).trim();
    String clientip = (String) args[0];
    int clientport = ((Integer)args[1]).intValue();
    String brushName = ((String)args[2]).trim();
    float floatval = ((Float)args[3]).floatValue();
    IncomingOSCqueue.add(new IncomingOSC(clientip.trim(), clientport,
      cmd.trim(), brushName.trim(), floatval));
  } else {
    println("UNKNOWN INCOMING OSC MESSAGE: " + addrstr);
  }
}

/* START DEMO CHANGE 12/10/2018 */
void processOSCevents() {
  while (IncomingOSCqueue.size() > 0) {
    IncomingOSC message = IncomingOSCqueue.poll();
    if (message != null) {
      println("CLIENT CONTACT! " + message.clientIP + ":" + message.clientPort
          + " : " + message.command + "," + message.paintbrush + "," + message.fparameter);
      if ("client".equals(message.command)) {
        println("DEBUG RECVD NEW CLIENT");
        sendBrushes(message.clientIP,message.clientPort);
        ClientIPset.add(new IncomingOSC(message.clientIP,message.clientPort,
          null,null,0.0));
      } else {
        PaintBrush brush = allshapes.get(message.paintbrush);
        if (brush == null) {
          println("ERROR, client send command '" + message.command + "' for invalid brush: "
            + message.paintbrush + " " + message.fparameter);
        } else {
          if (message.command.equals("addbrush")) {
            PaintBrush oldbrush = showshapes.put(message.paintbrush, brush);
            if (oldbrush == null) {
              for (IncomingOSC iosc : ClientIPset) {
                sendBrushes(iosc.clientIP, iosc.clientPort); // update active brushes
              }
            }
            brush.zap();
          } else if (message.command.equals("rembrush")) {
            boolean showing = (showshapes.containsKey(message.paintbrush));
            PaintBrush p = showshapes.get(message.paintbrush);
            if (p != null) {
              p.silenceNotes();
              forceSilence(p);
            }
            showshapes.remove(message.paintbrush);
            if (showing) {
              for (IncomingOSC iosc : ClientIPset) {
                sendBrushes(iosc.clientIP, iosc.clientPort); // update active brushes
              }
            }
          } else if (message.command.equals("Zap")) {
            for (PaintBrush nextShape : showshapes.values()) {
              nextShape.zap(); // zap them
            }
            forceSilence(null);
          } else if (message.command.equals("Erode")) {
            erodeMode = ! erodeMode ;
            dilateMode = false ;
          } else if (message.command.equals("Dilate")) {
            dilateMode = ! dilateMode ;
            erodeMode = false ;
          } else if (message.command.equals("Clear")) {
            background(backgroundColor);
          } else if (message.command.equals("Reset")) {
            background(backgroundColor);
            showshapes.clear();
            forceSilence(null);
          } else if (message.command.equals("Freeze")) {
            freezeMode = ! freezeMode ;
          } else if (message.command.equals("Move")) {
            moveMode = ! moveMode ;
          } else if (message.command.equals("Posterize")) {
            int plevel = constrain(round(message.fparameter),0,255) ;
            if (plevel == 0 || plevel > 1) {
              posterizeLevel = plevel ;
            }
          } else {
            if (showshapes.get(message.paintbrush) == null) {
              println("WARNING, client send command '" + message.command + "' for invisible brush: "
                + message.paintbrush + " " + message.fparameter);
            }
            brush.changeProperty(message.command, message.fparameter);
          }
        }
      }
    }
  }
}
/* END DEMO CHANGE 12/10/2018 */

void sendBrushes(String clientip, int clientport) {
  // println("DEBUG sendBrushes setup() runs in thread: " + Thread.currentThread().toString());
  NetAddress cliRemoteLocation = new NetAddress(clientip,clientport);
  for (String brname : allshapes.keySet()) {
    // println("DEBUG BRUSH: " + brname);
    String padname = brname ;
    while (padname.length() < paddedNameWidth) {
      padname += " ";
    }
    int visible = (showshapes.get(brname) != null) ? 1 : 0 ;
    OscMessage myMessage = new OscMessage("/paintdome18B/server");
    myMessage.add(padname);
    myMessage.add(new Integer(visible)); // brush is not visible; 1 would be visible
    oscP5.send(myMessage, cliRemoteLocation);
  }
}

// See http://midi.teragonaudio.com/tech/midispec.htm
// and https://www.midi.org/specifications/item/gm-level-1-sound-set
// See Parson's code at http://faculty.kutztown.edu/parson/spring2017/MidiKBSpring2016Parson.txt
HashMap<PaintBrush,Integer> brush2chan = new HashMap<PaintBrush,Integer>();
HashMap<Integer, PaintBrush> chan2brush = new HashMap<Integer, PaintBrush>();
int nextchan = 0 ;
void sendMIDI(PaintBrush brush, int opcode, int data1, int data2) {
  if (receiver == null) {
    return ;
  }
  Integer Chan = brush2chan.get(brush);
  int chan = -1 ;
  if (Chan != null) {
    chan = Chan.intValue();
  } else if (nextchan < 16) {
    brush2chan.put(brush, nextchan);
    chan2brush.put(nextchan, brush);
    chan = nextchan++;
  } else {
    // steal the least recently allocated
    // This may lead to stuck notes unless the PaintBrush object clear them.
    chan = nextchan % 16 ;
    nextchan++ ;
    PaintBrush oldy = chan2brush.get(chan);
    if (oldy != null) {
      oldy.silenceNotes(); // calls recursively into here, do that before remove().
      brush2chan.remove(oldy);
      brush2chan.put(brush, chan);
      chan2brush.put(chan, brush);
    }
  }
  //println("DEBUG ENTER sendMIDI " + opcode + "," + chan + "," + data1 + "," + data2);
  try {
    ShortMessage noteMessage = new ShortMessage() ;
    noteMessage.setMessage(opcode, chan, data1, data2);
    receiver.send(noteMessage, -1L);  // send it now
  } 
  catch (InvalidMidiDataException dx) {
    println("MIDI ERROR: " + dx.getMessage()); // Error messages go here.
  }
  //println("DEBUG EXIT sendMIDI");
}
/* START DEMO CHANGE 12/10/2018 */
void forceSilence(PaintBrush p) {
  final int AllNotesOff = 123 ;
  final int AllSoundsOff = 120 ;
  // See http://midi.teragonaudio.com/tech/midispec.htm
  if (p != null) {
    sendMIDI(p, ShortMessage.CONTROL_CHANGE, AllNotesOff, 0);
    sendMIDI(p, ShortMessage.CONTROL_CHANGE, AllSoundsOff, 0);
  } else {
    for (int chan = 0 ; chan < 16 ; chan++) {
      try {
        ShortMessage noteMessage = new ShortMessage() ;
        noteMessage.setMessage(ShortMessage.CONTROL_CHANGE, chan, AllNotesOff, 0);
        receiver.send(noteMessage, -1L);  // send it now
        noteMessage = new ShortMessage() ;
        noteMessage.setMessage(ShortMessage.CONTROL_CHANGE, chan, AllSoundsOff, 0);
        receiver.send(noteMessage, -1L);  // send it now
      } catch (InvalidMidiDataException dx) {
        println("MIDI ERROR: " + dx.getMessage()); // Error messages go here.
      }
    }
  }
}

String keycmd = "" ;
void keyPressed() {
  // println("DEBUG keyPressed setup() runs in thread: " + Thread.currentThread().toString());
  println("DEBUG keyPressed, keycmd = '" + keycmd + "', key = ' " + key + "'");
  if (key == CODED) {

  } else {
    if (key != '\n' && (key == '+' || key == '-' || key == 'P'
        || Character.isLowerCase(key)  // always start brush names with lower case
        || keycmd.length() >= 1)) {
      keycmd = keycmd + key ;
      println("DEBUG2 grows to " + keycmd);
    } else if (key == '\n') {
      println("DEBUG keycmd after newline is " + keycmd);
      if (keycmd.length() > 1 && (Character.isLowerCase(keycmd.charAt(0))
            || keycmd.charAt(0) == '+' || keycmd.charAt(0) == '-')) {
        if (keycmd.charAt(0) == '+') {
          PaintBrush thisshape = allshapes.get(keycmd.substring(1));
          if (thisshape != null) {
            println("DEBUG PUT: " + keycmd);
            showshapes.put(keycmd.substring(1), thisshape);
            thisshape.zap();
          } else {
            System.err.println("ERROR, no shape called " + keycmd.substring(1));
          }
        } else if (keycmd.charAt(0) == '-') {
          PaintBrush p = showshapes.get(keycmd.substring(1));
          if (p != null) {
            p.silenceNotes();
            forceSilence(p);
          }
          showshapes.remove(keycmd.substring(1));
        } else if (keycmd.charAt(0) == 'P') {
          posterizeLevel = Integer.parseInt(keycmd.substring(1));
        } else if (keycmd.trim().length() > 2) {
          // BRUSHNAME COMMAND FVAL
          float fval = 0 ;
          String floater = null ;
          keycmd = keycmd.trim();
          String [] fields = keycmd.split(" ");
          if (fields.length < 2) {
            println("ERROR, Invalid brush-keycmd: " + keycmd);
          } else {
            String brushname = fields[0];
            String cmd = null ;
            for (int i = 1 ; i < fields.length ; i++) {
              if (fields[i] != null && fields[i].length() > 0) {
                if (cmd == null) {
                  cmd = fields[i];
                } else if (floater == null) {
                  floater = fields[i];
                  try {
                    fval = Float.parseFloat(floater);
                  } catch (NumberFormatException fx) {
                    println("ERROR: Illegal float in: " + keycmd + ": "
                      + fx.getMessage());
                    brushname = cmd = floater = null ;
                  }
                  break ;
                }
              }
            }
            if (brushname != null && cmd != null) {
              PaintBrush thisshape = allshapes.get(brushname);
              if (thisshape == null) {
                println("ERROR, INVALID brush name: " + brushname + " in: "
                  + keycmd);
              } else if (! commandNames.contains(cmd)) {
                println("ERROR, INVALID keycmd: " + cmd + " in: "
                  + keycmd);
              } else {
                if (floater == null &&
                    (cmd.equals("scaleX") || cmd.equals("scaleY"))) {
                  fval = 1.0 ; // default for scaling is 1, not 0.
                }
                thisshape.changeProperty(cmd, fval);
              }
            } else {
              println("INVALID COMMAND: " + keycmd);
            }
          }
        }   
      }
      keycmd = "";
    } else if (keycmd.length() >= 1) {
      keycmd = keycmd + key ;
      println("DEBUG grows to " + keycmd);
    } else if (key == 'C') {
      background(backgroundColor);
    } else if (key == 'R') {
      background(backgroundColor);
      for (PaintBrush p : showshapes.values()) {
        p.silenceNotes();
      }
      showshapes.clear();
      forceSilence(null);
    } else if (key == 'F') {
      freezeMode = ! freezeMode ;
    } else if (key == 'M') {
      moveMode = ! moveMode ;
    } else if (key == 'D') {
      dilateMode = ! dilateMode ;
      /*posterMode = */ erodeMode = false ;
    } else if (key == 'E') {
      erodeMode = ! erodeMode ;
      dilateMode = /* posterMode = */ false ;
    } else if (key == '~') {
      for (PaintBrush nextShape : showshapes.values()) {
        nextShape.configure(); // shuffle them
      }
    } else if (key == 'Z') {
      for (PaintBrush nextShape : showshapes.values()) {
        nextShape.zap(); // zap them
      }
      forceSilence(null);
    }
  }
}
/* END DEMO CHANGE 12/10/2018 */

PImage loadScaleImage(String filename) {
  final int limit = 100 ;  // max 100 pixels in either direction
  PImage result = loadImage(filename);
  if (result == null) {
    println("ERROR, no such image file: " + filename);
    exit(); // TODO, do not exit in final version!!!
  } else {
    float aspectRatio = float(result.width) / float(result.height);
    if (result.width >= result.height) {
      result.resize(limit, round(limit/aspectRatio));
    } else {
      result.resize(round(limit * aspectRatio), limit);
    }
  }
  return result ;
}

void setupPaintBrushes() { 
  allshapes.clear();
  showshapes.clear();
  allshapes.put("mpate337", new mpate337());
  allshapes.put("calle393", new calle393());
  allshapes.put("jbach633", new jbach633());
  allshapes.put("sbres531", new sbres531());
  allshapes.put("mcarh409", new mcarh409());
  allshapes.put("dcarm213", new dcarm213());
  allshapes.put("bcres539", new bcres539());
  allshapes.put("mdial558", new mdial558());
  allshapes.put("ddoug969", new ddoug969());
  allshapes.put("jfish787", new jfish787());
  allshapes.put("jgang681", new jgang681());
  allshapes.put("tgang529", new tgang529());
  allshapes.put("pgeev638", new pgeev638());
  allshapes.put("sharb791", new sharb791());
  allshapes.put("ehise277", new ehise277());
  allshapes.put("tjenn300", new tjenn300());
  allshapes.put("rkrei300", new rkrei300());
  allshapes.put("anunz875", new anunz875());
  allshapes.put("apipe845", new apipe845());
  allshapes.put("nschn741", new nschn741());
  allshapes.put("dvera889", new dvera889());
  // LATE allshapes.put("twalr073", new twalr073());
  allshapes.put("rward252", new rward252());
  allshapes.put("wzieg033", new wzieg033());
  allshapes.put("nzoln043", new nzoln043());
  allshapes.put("parson", new parson());
}

// This interface PaintBrush specifies the functions STUDENTS have to write.
// Class parson shows one possible class. You must customize your own.
// IMPORTANT: The draw() function does a translate(width/2, height/2)
// so that 0,0 is at the center (top) of the planetarium dome.
// Your x and y coordinates may range from -radius to +radius; radius
// is a global variable that is the radius of the planetarium dome.
// See class parson's display() and move() functions to understand this.
interface PaintBrush {
  // configure sets some of your objects data fields to constants or random values.
  // Avoid going outside -radius to +radius in x and y!
  // Do not go outside 0..360 in hue. We are using the Hue-Saturation-Brightness color model.
  // display() uses pushMatrix() and translate(x,y) as previous display() functions.
  // Its final command should be a paired popMatrix(). See parson.display() for an example.
  // display() MUST use a PImage loaded by the constructor via loadScaleImage(FILENAME).
  // display must use an alpha value derived from some property of location, speed,
  // time, etc. to fade the image in & out, revealing a graphic shape colored using
  // HSB mode; the shape plots before the image and is smaller than the shape so that
  // the image must completely cover the shape when alpha is 100%.
  void display();
  // Move adjusts location and speed variables for x, y, hue, and other data fields.
  void move();
  // changeProperty changes one of the properties sent via OSC, for example datagram:
  //          set speedX (+ is right, - is left)
  //            /paintdome18B/speedX brushname floatval
  // translates into these parameters on changeProperty, where brushname identifies this object
  //          name="speedX", value=floatval
  void changeProperty(String name, float value) ;
  // zap sets properties to their default values
  void zap();
  // configure() randomizes some properties
  void configure();
  // silence any in-play MIDI notes.
  void silenceNotes();
  // Return the name of this class object, such as "parson" for class parson.
  String getName();
}

/* Class parson is a working example class but it is incomplete because
/* display() does not use all of the state variables set via OSC commands.
/* Your class must use them all!
/* UPDATED 12/10/2018 FULLY FUNCTIONAL
*/
class parson implements PaintBrush {
  PImage img = loadScaleImage("junior.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  parson() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use them all!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX));
    shearY(radians(shearY));
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
        imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // once every second
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 12, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "parson";
  }
}

class jfish787 implements PaintBrush {
  PImage img = loadScaleImage("jfish787_image.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float speedXS = 0.0;
  float scaleXLB = 0.5 ;
  float speedXLB = 0.5;// Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float speedXUB = 2.0;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float speedYS = 0.0;
  float scaleYLB = 0.5 ;
  float speedYLB = 0.5;
  float scaleYUB = 2.0 ;
  float speedYUB = 2.0;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  jfish787() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(scaleX,scaleY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(scaleX,scaleY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,(.8*img.width),(.8*img.height));
   // ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 56, 0); // 57 is trumpet minus 1
      // PAN stereo based on X location
      final int minorscale = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, 95, 65);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 36 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
     scaleX += speedX ;
    if (scaleX <= -radius) {
      scaleX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (scaleX >= radius) {
      scaleX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    scaleXS += speedXS ;
    if (scaleXS <= -radius) {
      scaleXS = (-radius)+1 ;
      speedXS = abs(speedXS);
    } else if (scaleXS >= radius) {
      scaleXS = radius - 1 ;
      speedXS = - abs(speedXS) ;
    }
    scaleXLB += speedXLB ;
    if (scaleXLB <= -radius) {
      scaleXLB = (-radius)+1 ;
      speedXLB = abs(speedXLB);
    } else if (scaleXLB >= radius) {
      scaleXLB = radius - 1 ;
      speedXLB = - abs(speedXLB) ;
    }
    scaleXUB += speedXUB ;
    if (scaleXUB <= -radius) {
      scaleXUB = (-radius)+1 ;
      speedXUB = abs(speedXUB);
    } else if (scaleXUB >= radius) {
      scaleXUB = radius - 1 ;
      speedXUB = - abs(speedXUB) ;
    }
      scaleY += speedY ;
    if (scaleY <= -radius) {
      scaleY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (scaleY >= radius) {
      scaleY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
      scaleYS += speedYS ;
    if (scaleYS <= -radius) {
      scaleYS = (-radius)+1 ;
      speedYS = abs(speedYS);
    } else if (scaleYS >= radius) {
      scaleYS = radius - 1 ;
      speedYS = - abs(speedYS) ;
    }
    scaleYLB += speedYLB ;
    if (scaleYLB <= -radius) {
      scaleYLB = (-radius)+1 ;
      speedYLB = abs(speedYLB);
    } else if (scaleYLB >= radius) {
      scaleYLB = radius - 1 ;
      speedYLB = - abs(speedYLB) ;
    }
     scaleYUB += speedYUB ;
    if (scaleYUB <= -radius) {
      scaleYUB = (-radius)+1 ;
      speedYUB = abs(speedYUB);
    } else if (scaleYUB >= radius) {
      scaleYUB = radius - 1 ;
      speedYUB = - abs(speedYUB) ;
    }
      rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    } 
      shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
     shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
   }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "jfish787";
  }
}

class ehise277 implements PaintBrush {
  PImage img = loadScaleImage("buffdolphin.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  ehise277() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    // int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    int alpha = int(50 - (50 * dist(scaleX,scaleY,0,0) / radius))+25 ; // 25 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(shearX);
    shearY(shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // hide geometric shape behind image
    noStroke();
    colorMode(HSB, 100);
      for (int i = 0; i < 100; i++) {
        for (int j = 0; j < 100; j++) {
        stroke(i, j, 100);
        rect(i, j, i, j);
        }
    }  
    
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0); 
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 81, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      final int tremelocontroller = 95;
      // pan based on percentage left-to-right:
      int tremelo = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, tremelocontroller, tremelo);
      //int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      //sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = minorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    shearX += shearXS ;
    if (shearX <= shearXB) {
      shearX = shearXB+1 ;
      shearXS = abs(shearXS);
    } else if (shearX >= shearXB) {
      shearX = shearXB - 1 ;
      shearXS = - abs(shearXS) ;
    }
    shearY += shearYS ;
    if (shearY <= shearYB) {
      shearY = shearYB+1 ;
      shearYS = abs(shearYS);
    } else if (shearY >= shearYB) {
      shearY = shearYB - 1 ;
      shearYS = - abs(shearYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  void changeProperty(String name, float value) {
     println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
     for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "ehise277";
  }
}
class rward252 implements PaintBrush {
  PImage img = loadScaleImage("brassard.jpg");
  // CPU-eating experiment PShape shp = loadShape("logan.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  rward252() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    float alphaRate = frameRate * 5; // Cycle in and out of transparent every alphaRate seconds
    alpha = int((frameCount % alphaRate) / alphaRate * 255);
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(shearX);
    shearY(shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    rectMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 58, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      final int chorus = 93;
      // pan based on percentage left-to-right:
      //int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      int chorus0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, chorus, chorus0to127);
      //sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 14 * int(random(3,6)); // change from 12 to 14 
      lastnotes[nextnote] = minorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "rward252";
  }
}
class calle393 implements PaintBrush {
PImage img = loadScaleImage("nyan.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
   calle393() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    float alphaRate = frameRate * 5;
    alpha = int((frameCount % alphaRate) / alphaRate * 255 );
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    rectMode(CENTER);
    shearX(radians(shearX));
    shearY(radians(shearY));
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 32
      , 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller , pan0to127 - 63);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    //ScaleY
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
      
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
      silenceNotes();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "calle393";
  }
}


class tgang529 implements PaintBrush {
  PImage img = loadScaleImage("earth.png");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  tgang529() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  void display() {
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    translate(shearX,shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 51, 0); // 51 is synth
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      //int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, 91);//pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "tgang529";
  }
}
class bcres539 implements PaintBrush {
  //polygon function taken from Processing Website
  void polygon(float x, float y, float radius, int npoints) {
  float angle = TWO_PI / npoints;
  beginShape();
  for (float a = 0; a < TWO_PI; a += angle) {
    float sx = x + cos(a) * radius;
    float sy = y + sin(a) * radius;
    vertex(sx, sy);
  }
  endShape(CLOSE);
}
 //PImage img = loadScaleImage("arcReactor.png"); //TEST
 PImage img = loadScaleImage("ocean.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  bcres539() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    float alphaRate = frameRate * 5;
    int alpha = int((frameCount % alphaRate) / alphaRate * 255) ;
    //int alpha = int((abs(scaleX)+abs(scaleY)/2)+50); // 50 to 100%
    hue = (hue + 1) % 360;
    //hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(shearX);
    shearY(shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
     stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    fill(hue, 100, 100, 100);
    polygon(0,0, 35, 6);
    //ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    //triangle(0,-round((.8*img.height)/2) round((.8*img.width)/2),
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 66, 0); // 66 is Tenor Sax
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    //shearX
    shearX += shearXS ;
    if (shearXB != 0) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    
    //shearY
    shearY += shearYS ;
    if (shearYB != 0) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
    //sheary += shearYS;
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "bcres539";
  }
}
class ddoug969 implements PaintBrush {
  PImage img = loadScaleImage("ddoug969.png");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  ddoug969() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 28, 0);
      // PAN stereo based on X location
      final int effectcontroller = 91 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int effect0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, effectcontroller, effect0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  } //end of display
  
  
  
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    shearX += shearXS ;
    if (shearX <= shearXB) {
      shearX = shearXB+1 ;
      shearXS = abs(shearXS);
    } else if (shearX >= shearXB) {
      shearX = shearXB - 1 ;
      shearXS = - abs(shearXS) ;
    }
    shearY += shearYS ;
    if (shearY <= shearYB) {
      shearY = shearYB+1 ;
      shearYS = abs(shearYS);
    } else if (shearY >= shearYB) {
      shearY = shearYB - 1 ;
      shearYS = - abs(shearYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "ddoug969";
  }
}
class nschn741 implements PaintBrush {
    PImage img = loadScaleImage("NyanCat.png");
 // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  nschn741() {
     accents[0] = accents[1] = accents[5] = 1.0 ; 
    accents[2] = accents[3] = accents[7] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(50 - (50 * dist(scaleX,scaleY,0,0) / radius))+100 ;
    int alpha = int(75 - (75 * dist(scaleX,scaleY,0,0) / radius))+75 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX));
    shearY(radians(shearY));
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(1.2*img.width),round(1*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 91, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      
      
      final int phaserController = 95;
      int phaser0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, phaserController, phaser0to127);

   
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    
    scaleY += scaleYS ;
    if (scaleX <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
   
    
  
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
   
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    
   shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    
    
     shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
   
    
    
    
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "nschn741";
  }
}
class rkrei300 implements PaintBrush {
  PImage img = loadScaleImage("apollo.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  rkrei300() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }

  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    float alpha = (360-(speedRot*3.6)) ; // decreses alpha as speedrot increases

    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(shearX);
    shearY(shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.6*img.width),round(.6*img.height));
  //  ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE,58, 0); // 12 is marimba
      // PAN stereo based on X location
      final int volumecontroller = 7 ; //  http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, volumecontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
        scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    
       shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    
         shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }  
    
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "rkrei300";
  }
}
class wzieg033 implements PaintBrush {
  PImage img = loadScaleImage("logo.png");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;

  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;

  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;

  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
    wzieg033() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  void zap() {
      setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();

  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-25,25);
    speedY = random(-17,20);
    scaleX = 1.0;
    scaleXS = random(-16,12);
    scaleXLB = random(-7,20);
    scaleXUB = random(-10,20);
    scaleY = 1.0;
    scaleYS = random(-20,10);
    scaleYLB = random(-5,5);
    scaleYUB = random(-8,20);
    rotate = random(-15,15);
    speedRot = random(-13,17);
    rotateB = noRotateB;

    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;

  }
  void display() {
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
   int  alpha = int(rotate*.71);  
 //   int alpha = int(50 - (50 * dist(scaleX,scaleY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    scale(scaleX,scaleY);
    rotate(radians(rotate));
    shearX(shearX);
    shearY(shearY);

    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.5*img.width),round(.5*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    image(img,0,0);
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 0, 0); // 12 is marimba
      // PAN stereo based on X location
      final int volumecontroller = 7 ; // http://midi.teragonaudio.com/tech/midispec.htm
      //final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int volume0to127 = constrain(round(((setY+radius) / diameter) * 127),0,127);
      //int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, volumecontroller, volume0to127);
      //sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  void move() {
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }

    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
     rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }

    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }

    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }

  }
  void changeProperty(String name, float value) {
  println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }

  }
  void silenceNotes() {
  for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;

  }
  String getName() {
    return "wzieg033";
  }
}

class jgang681 implements PaintBrush {
  PImage img = loadScaleImage("boy.jpg");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  jgang681() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(20 - (20 * dist(setX,setY,0,0) / radius))+80 ;
    int alpha = int(55 - (55 * dist(setX,setY,0,0) / radius))+45 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    shearX(radians(shearX));
    shearY(radians(shearY));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 27, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    
    shearX += speedRot ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        speedRot = abs(speedRot);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    
    shearY += speedRot ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        speedRot = abs(speedRot);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "jgang681";
  }
}

class dcarm213 implements PaintBrush {
  PImage img = loadScaleImage("girardi.jpg");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 1.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 4.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 1.2 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.7 ;
  float scaleY = 4.0 ;
  float scaleYS = 1.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.3 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  dcarm213() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (60 * speedY / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    rectMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0, 0, 80, 40); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 37, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      final int phaserControl = 70;
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      int phase0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, phaserControl, phase0to127);
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    
    if (shearX < -shearXB) {
      shearX = (-shearXB)+1 ;
      shearXS = abs(shearXS);}
    else if (shearX > -shearXB) {
      shearX = (-shearXB)+1 ;
      shearXS = abs(shearXS);}
      
    if (shearY < -shearYB) {
      shearY = (-shearYB)+1 ;
      shearYS = abs(shearYS);}
    else if (shearY > -shearYB) {
      shearY = (-shearYB)+1 ;
      shearYS = abs(shearYS);}
      
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "dcarm213";
  }
}
class mdial558 implements PaintBrush {
 
  PImage img = loadScaleImage("imgfinal.png");
 
 // PImage img = loadScaleImage("junior.png");
  // CPU-eating experiment PShape shp = loadShape("#FF0000");
  
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  mdial558() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
   
    silenceNotes();
    
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
   
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    
   // shearX = 0.0 ;
   // shearXS = 0.0 ;
   // shearXB = noRotateB ;
    //shearY = 0.0 ;
   // shearYS = 0.0 ;
   // shearYB = noRotateB ;
   
   
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
  }
  
  void display() {
    
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    
    translate(setX,setY);
    
    rotate(radians(rotate));
   
    shearX(radians(shearX));
    shearY(radians(shearY));
    
     scale(scaleX,scaleY);
    
   
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    
    image(img,0,0);
    
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 12, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
     
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
       
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
      
       }
    popMatrix();
  }
  
  
  void move() {
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    
    
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
   
    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -rotateB) {
        shearX = (-rotateB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > rotateB) {
        shearX = rotateB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -rotateB) {
        shearY = (-rotateB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > rotateB) {
        shearY = rotateB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
    
  }
  
  void changeProperty(String name, float value) {
     println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  
  }
  
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
    
  }
  String getName() {
    return "mdial558";
  }
}
class nzoln043 implements PaintBrush {
  PImage img = loadScaleImage("faces.jpg");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = -20.0 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 20.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = -20.0 ;
  float scaleYUB = 20.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  //int midiCounter = int(frameCount + frameRate);
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  nzoln043() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = -20.0 ;
    scaleXUB = 20.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = -20.0 ;
    scaleYUB = 20.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    float alphaRate = frameRate * 50 ;
    alpha = int((frameCount % alphaRate) / alphaRate * 255) ;
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX));
    shearY(radians(shearY));
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    //ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    triangle(-round(.8*img.width)/3,round(.8*img.height)/3,0,-round(.8*img.height)/3,round(.8*img.width)/3,round(.8*img.height)/3);
    //ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 43, 0); // 12 is marimba
      // volume stereo based on X location
      final int volumecontroller = 7 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int volume0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, volumecontroller, volume0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "nzoln043";
  }
}
class mcarh409 implements PaintBrush {
  PImage img = loadScaleImage("pepe.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  mcarh409() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(10,-10); // randomize most but not all state fields
    setY = random(10,-10);
    speedX = random(10,-10);
    speedY = random(10,-10);
    scaleX = 1.0;
    scaleXS = random(10,-10);
    scaleXLB = random(10,-10);
    scaleXUB = random(10,-10);
    scaleY = 1.0;
    scaleYS = random(10,-10);
    scaleYLB = random(10,-10);
    scaleYUB = random(10,-10);
    rotate = random(10,-10);
    speedRot = random(10,-10);
    rotateB = noRotateB;
    shearX = random(10,-10);
    shearXS = random(10,-10);
    shearXB = random(10,-10);
    shearY = random(10,-10);
    shearYS = random(10,-10);
    shearYB = random(10,-10);
    
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 106, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 50 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "mcarh409";
  }
}
class anunz875 implements PaintBrush {
 PImage img = loadScaleImage("Steampic.jpg");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  anunz875() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(20 - (20 * dist(shearX,shearY,0,0) / radius))+80 ;
    int alpha = int(30 - (30 * dist(shearX,shearY,0,0) / radius))+70 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    shearX(radians(shearX));
    shearY(radians(shearY));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0, round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 111, 0); // 111 is fiddle
      // PAN stereo based on X location
      final int pancontroller = 94 ; // http://midi.teragonaudio.com/tech/midispec.htm
      
      // celeste based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
   
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    
     scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    
     shearX += speedRot ;
    if (shearX != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        speedRot = abs(speedRot);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    
    shearY += speedRot ;
    if (shearY != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        speedRot = abs(speedRot);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "anunz875";
  }
}
class mpate337 implements PaintBrush {
  PImage img = loadScaleImage("twins.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  mpate337() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(scaleX,scaleY,0,0) / radius))+75 ;
    int alpha = int(95 - (95 * dist(rotate,rotate,0,0) / radius))+5 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX));
    shearX(radians(shearY));
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    //ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    rect(0,0,round(.8*img.width),round(.8*img.height));
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 28, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 91 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    shearX += speedRot ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        speedRot = abs(speedRot);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    shearY += speedRot ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        speedRot = abs(speedRot);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "mpate337";
  }
}

class  pgeev638 implements PaintBrush {
  PImage img = loadScaleImage("color_wheel.png");
  float hue =1;
  float speedhue = 1 ;
  float setX = 0.0 ;    
  float setY = 0.0 ;    
  float speedX = 0.0 ;  
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 1.0 ;  
  float scaleXUB = 1.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 1.0 ;
  float scaleYUB = 1.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; 
  int nextnote = 0 ; 
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; 
  int countdown = countdownRate ; 
  float [] accents = new float [ 8 ] ; 
  int accentix = 0 ;
   pgeev638() {
    accents[0] = accents[3] = accents[6] = 1.0 ; 
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
 
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; 
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); 
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  
  void display() {
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; 
    float alphaRate = frameRate * 5;
    alpha = int ((frameRate % alphaRate) / alphaRate * 255);
    hue = (hue + 1) % 360 ; 
    pushMatrix();
    rotate(radians(rotate));
    translate(setX,setY);
    scale(scaleX,scaleY);
    shearX(shearX);
    shearY(shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    rectMode(CENTER);
    
    stroke(hue, 100, 100, 100);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.8*img.width),round(.8*img.height)); 
    tint(0,0,100,round(alpha));
    image(img,0,0);
    
    if (--countdown < 2) {
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 0, 0); 
      final int pancontroller = 5 ;
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); 
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  void move() {
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }

    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
     
    //scaleY
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
      
    }
    
    //shearX
    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
  
  
      //shearY
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; 
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; 
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ;
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return " pgeev638";
  }
}
class jbach633 implements PaintBrush {
PImage img = loadScaleImage("Hat.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  jbach633() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX));
    shearY(radians(shearY));
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 12, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    
       shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    
        
       shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearX = (-shearYB)+1 ;
        shearXS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "jbach633";
  }
}

class sharb791 implements PaintBrush {
  PImage img = loadScaleImage("coolDog.jpg");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
    sharb791() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
    }
  
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
     setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  void display() {
    //int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    float alphaRate = frameRate * 5;
    alpha = int((frameCount % alphaRate) / alphaRate * 255);
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(shearX);
    shearY(shearY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    rectMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 69, 0); // Change to a harmonica
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 69 ;
      //* int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      //println("Debug" + );
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],round( accents[accentix] * 127)/*lastnotes[nextnote],
        round(midilevel*accents[accentix])*/); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  void move() {
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  
  shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
     for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "sharb791";
  }
}
class apipe845 implements PaintBrush {     // modified from class parson.
  PImage img = loadScaleImage("butterscotch.png");
 
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  apipe845() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on scaling.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(scaleX,scaleY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX));
    shearY(radians(shearY));
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
      imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    stroke(hue, 80, 80, 80 /* round(.5*(100-alpha))*/);
    strokeWeight(9);
    noFill();
    rect(0,0,round(.5*img.width),round(.5*img.height)); // .5 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 10, 0); // 10 is musicbox
      // PAN stereo based on X location
      final int effectscontroller = 91 ; // http://midi.teragonaudio.com/tech/midispec.htm
      int effects0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, effectscontroller, effects0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(2,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ; // setX
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ; //setY
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ; //scaleX
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    
     scaleY += scaleYS ; //scaleY
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ; //rotate
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    shearX += shearXS ; //shearX
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX >shearXS) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
     shearY += shearYS ; //shearY
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY >shearYS) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    
 
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    
    
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "apipe845";
  }
}
class dvera889 implements PaintBrush {
  PImage img = loadScaleImage("MockingSpongeBob.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 36.0 ;  // configure() can adjust a subset of them.
  float speedY = 36.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 1.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  dvera889() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 9.0 ;
    speedY = 1.0 ;
    scaleX = 1.0 ;
    scaleXS = 9.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 4.0 ;
    speedRot = 9.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(setX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(setX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    ellipse(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 12, 0); // 12 is marimba
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "dvera889";
  }
}
class sbres531 implements PaintBrush {
  PImage img = loadScaleImage("sunburn1.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 0.5 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 2.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 0.5 ;
  float scaleYUB = 2.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  sbres531() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 0.5 ;
    scaleXUB = 2.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 0.5 ;
    scaleYUB = 2.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(shearX,setY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(shearX,setY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    shearX(radians(shearX));
    shearY(radians(shearY));
    scale(scaleX,scaleY);
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.8*img.width),round(.8*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 10, 0); // 
      // PAN stereo based on X location
      final int changecontroller = 8 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int change0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, changecontroller, change0to127);
      
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  }
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
     scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
     shearY += shearYS ;
    if (shearYB != shearYB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearX > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
      while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
    }
    shearX += shearXS ;
    if (shearXB != shearXB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
      while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "sbres531";
  }
}

class tjenn300 implements PaintBrush {
  PImage img = loadScaleImage("spongebob.png");
  //PImage img = loadScaleImage("junior.png");
  // CPU-eating experiment PShape shp = loadShape("junior.svg");
  float hue, speedhue = 1 ;
  float setX = 0.0 ;    // These are the state variables. display() must use them all.
  float setY = 0.0 ;    // move() must adjust the ones driven by speeds & bounds.
  float speedX = 0.0 ;  // configure() can adjust a subset of them.
  float speedY = 0.0 ;
  float scaleX = 1.0 ;
  float scaleXS = 0.0 ;
  float scaleXLB = 1.0 ;  // Parson set a few bounds here. STUDENTs must set their own bounds.
  float scaleXUB = 5.0 ;
  float scaleY = 1.0 ;
  float scaleYS = 0.0 ;
  float scaleYLB = 1.0 ;
  float scaleYUB = 5.0 ;
  float rotate = 0.0 ;
  float speedRot = 0.0 ;
  float rotateB = noRotateB ;
  float shearX = 0.0 ;
  float shearXS = 0.0 ;
  float shearXB = noRotateB ;
  float shearY = 0.0 ;
  float shearYS = 0.0 ;
  float shearYB = noRotateB ;
  int [] lastnotes = {-1, -1, -1} ; // MIDI starts out silent
  int nextnote = 0 ; // index into lastnotes
  int scaleindex = 0 ;
  final int defaultCountdownRate = 120 ;
  int countdownRate = defaultCountdownRate ; // count down until the next note
  int countdown = countdownRate ; // music tempo varies with speedX & speedY
  float [] accents = new float [ 8 ] ; // 8 beats to the measure for now
  int accentix = 0 ;
  tjenn300() {
    accents[0] = accents[3] = accents[6] = 1.0 ; // note-note-rest-note-note-rest-note-rest for now
    accents[1] = accents[4] = accents[6] = 0.75 ;
  }
  // set propertties back to stationary defaults
  void zap() {
    setX = 0.0 ;
    setY = 0.0 ;
    speedX = 0.0 ;
    speedY = 0.0 ;
    scaleX = 1.0 ;
    scaleXS = 0.0 ;
    scaleXLB = 1.0 ;
    scaleXUB = 5.0 ;
    scaleY = 1.0 ;
    scaleYS = 0.0 ;
    scaleYLB = 1.0 ;
    scaleYUB = 5.0 ;
    rotate = 0.0 ;
    speedRot = 0.0 ;
    rotateB = noRotateB ; // no bound on rotation & related shears
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
    silenceNotes();
  }
  void configure() {
    setX = random(-10,10); // randomize most but not all state fields
    setY = random(-10,10);
    speedX = random(-10,10);
    speedY = random(-10,10);
    scaleX = 1.0;
    scaleXS = random(-10,10);
    scaleXLB = random(-10,10);
    scaleXUB = random(-10,10);
    scaleY = 1.0;
    scaleYS = random(-10,10);
    scaleYLB = random(-10,10);
    scaleYUB = random(-10,10);
    rotate = random(-10,10);
    speedRot = random(-10,10);
    rotateB = noRotateB;
    /*
    shearX = random(-10,10);
    shearXS = random(-10,10);
    shearXB = random(-10,10);
    shearY = random(-10,10);
    shearYS = random(-10,10);
    shearYB = random(-10,10);
    */
    // I later decided to zap the shears and force them only manually:
    shearX = 0.0 ;
    shearXS = 0.0 ;
    shearXB = noRotateB ;
    shearY = 0.0 ;
    shearYS = 0.0 ;
    shearYB = noRotateB ;
  }
  /* parson.display() is a working example but it is incomplete because
  /* display() does not use all of the state variables set via OSC commands.
  /* Your class must use all of the location variables, e.g., location
  /* in shearX space.!
  */
  void display() {
    // My image alpha ranges from 75 to 100%, based on distance from center.
    int midilevel = int(25 - (25 * dist(scaleX,scaleY,0,0) / radius))+75 ;
    int alpha = int(50 - (50 * dist(scaleX,scaleY,0,0) / radius))+50 ; // 50 to 100%
    hue = (hue + 1) % 360 ; // hue sets the color of the shape(s) behind the image.
    pushMatrix();
    translate(setX,setY);
    rotate(radians(rotate));
    scale(scaleX,scaleY);
    shearX(radians(shearX)); //edited by me
    shearY(radians(shearY)); //edited by me
    textMode(SHAPE);
    textAlign(CENTER, CENTER);
    imageMode(CENTER);
    shapeMode(CENTER);
    ellipseMode(CENTER);
    // Stroke hue used for my ellipse, no fill.
    stroke(hue, 100, 100, 100 /* round(.5*(100-alpha))*/);
    strokeWeight(10);
    noFill();
    rect(0,0,round(.7*img.width),round(.7*img.height)); // .8 hides behind image
    tint(0,0,100,round(alpha)); // Don't color the image, just fade it.
    // Saturation of 0 ignores hue; brightness is 100%; alpha derived above.
    image(img,0,0);
    // DO THE MIDI THING:
    if (--countdown < 2) {
      // countdownRate determines how fast notes are played
      if (lastnotes[nextnote] != -1) {
        sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[nextnote], 0);
      }
      sendMIDI(this, ShortMessage.PROGRAM_CHANGE, 3, 0); // 11
      // PAN stereo based on X location
      final int pancontroller = 10 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // pan based on percentage left-to-right:
      int pan0to127 = constrain(round(((setX+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, pancontroller, pan0to127);
      
      final int volumecontroller = 7 ; // http://midi.teragonaudio.com/tech/midispec.htm
      // volume based on percentage left-to-right:
      int volume0to127_2 = constrain(round(((setY+radius) / diameter) * 127),0,127);
      sendMIDI(this, ShortMessage.CONTROL_CHANGE, volumecontroller, volume0to127_2);
      
      // See https://en.wikipedia.org/wiki/General_MIDI, start at 0 by subtracting 1.
      int myoctave = 12 * int(random(3,6));
      lastnotes[nextnote] = majorscale[scaleindex] + myoctave;
      sendMIDI(this, ShortMessage.NOTE_ON, lastnotes[nextnote],
        round(midilevel*accents[accentix])); // use midilevel for volume
      accentix = (accentix+1) % accents.length ;
      nextnote = (nextnote+1) % lastnotes.length ;
      scaleindex = int(random(0,majorscale.length));
      if (speedX != 0 || speedY != 0) {
        countdownRate = constrain(round(float(defaultCountdownRate) / sqrt(speedX * speedX + speedY * speedY)),10,120);
      }
      countdown = countdownRate ;
    }
    popMatrix();
  } //end display
  // STUDENT: move() must utilize *ALL* of the state variables
  // that relate to the brush commands, updating and range-checking
  // the appropriate ones. The missing ones have almost identical
  // counterparts, e.g., the rotation variables take the same kind
  // of values and the limits as the shearing variables, both of
  // which are specified in degrees in this assignment.
  void move() {
    // STUDENT: Make sure to use *ALL* of the appropriate state variables.
    setX += speedX ;
    if (setX <= -radius) {
      setX = (-radius)+1 ;
      speedX = abs(speedX);
    } else if (setX >= radius) {
      setX = radius - 1 ;
      speedX = - abs(speedX) ;
    }
    setY += speedY ;
    if (setY <= -radius) {
      setY = (-radius)+1 ;
      speedY = abs(speedY);
    } else if (setY >= radius) {
      setY = radius - 1 ;
      speedY = - abs(speedY) ;
    }
    // image is transparent at the periphery
    hue += speedhue ;
    if (hue < 0 || hue >= 360) {
      speedhue = - speedhue ;
      hue = constrain(hue,0,359);
    }
    scaleX += scaleXS ;
    if (scaleX <= scaleXLB) {
      scaleX = scaleXLB+1 ;
      scaleXS = abs(scaleXS);
    } else if (scaleX >= scaleXUB) {
      scaleX = scaleXUB - 1 ;
      scaleXS = - abs(scaleXS) ;
    }
    scaleY += scaleYS ;
    if (scaleY <= scaleYLB) {
      scaleY = scaleYLB+1 ;
      scaleYS = abs(scaleYS);
    } else if (scaleY >= scaleYUB) {
      scaleY = scaleYUB - 1 ;
      scaleYS = - abs(scaleYS) ;
    }
    rotate += speedRot ;
    if (rotateB != noRotateB) {
      if (rotate < -rotateB) {
        rotate = (-rotateB)+1 ;
        speedRot = abs(speedRot);
      } else if (rotate > rotateB) {
        rotate = rotateB-1 ;
        speedRot = - abs(speedRot) ;
      }
    }
    while (rotate < 0) {
      rotate += 360.0 ;
    }
    while (rotate > 360.0) {
      rotate -= 360.0 ;
    }
    shearX += shearXS ;
    if (shearXB != noRotateB) {
      if (shearX < -shearXB) {
        shearX = (-shearXB)+1 ;
        shearXS = abs(shearXS);
      } else if (shearX > shearXB) {
        shearX = shearXB-1 ;
        shearXS = - abs(shearXS) ;
      }
    }
    while (shearX < 0) {
      shearX += 360.0 ;
    }
    while (shearX > 360.0) {
      shearX -= 360.0 ;
    }
    shearY += shearYS ;
    if (shearYB != noRotateB) {
      if (shearY < -shearYB) {
        shearY = (-shearYB)+1 ;
        shearYS = abs(shearYS);
      } else if (shearY > shearYB) {
        shearY = shearYB-1 ;
        shearYS = - abs(shearYS) ;
      }
    }
    while (shearY < 0) {
      shearY += 360.0 ;
    }
    while (shearY > 360.0) {
      shearY -= 360.0 ;
    }
  }
  // changeProperty should be complete as supplied. move() needs to be enhanced.
  // The Android client calls changeProperty indirectly via OSC/UDP.
  void changeProperty(String name, float value) {
    println("DEBUG changeProperty: " + name + "," + value);
    if (name.equals("reset")) {
      zap();
    } else if (name.equals("shuffle")) {
      configure();
    } else if (name.equals("setX")) {
      setX = value ;
    } else if (name.equals("setY")) {
      setY = value ;
    } else if (name.equals("speedX")) {
      speedX = value ;
    } else if (name.equals("speedY")) {
      speedY = value ;
    } else if (name.equals("scaleX")) {
      scaleX = value ;
    } else if (name.equals("scaleXS")) {
      scaleXS = value ;
    } else if (name.equals("scaleXLB")) {
      scaleXLB = value ;
    } else if (name.equals("scaleXUB")) {
      scaleXUB = value ;
    } else if (name.equals("scaleY")) {
      scaleY = value ;
    } else if (name.equals("scaleYS")) {
      scaleYS = value ;
    } else if (name.equals("scaleYLB")) {
      scaleYLB = value ;
    } else if (name.equals("scaleYUB")) {
      scaleYUB = value ;
    } else if (name.equals("rotate")) {
      rotate = value ;
    } else if (name.equals("speedRot")) {
      speedRot = value ;
    } else if (name.equals("rotateB")) {
      if (value > 99) {
        rotateB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        rotateB = value ;
      }
    } else if (name.equals("shearX")) {
      shearX = value ;
    } else if (name.equals("shearXS")) {
      shearXS = value ;
    } else if (name.equals("shearXB")) {
      if (value > 99) {
        shearXB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearXB = value ;
      }
    } else if (name.equals("shearY")) {
      shearY = value ;
    } else if (name.equals("shearYS")) {
      shearYS = value ;
    } else if (name.equals("shearYB")) {
      if (value > 99) {
        shearYB = noRotateB ; // tablet menu has no entry for noRotateB
      } else {
        shearYB = value ;
      }
    }
  }
  void silenceNotes() {
    for (int n = 0 ; n < lastnotes.length ; n++) {
        if (lastnotes[n] >= 0 && lastnotes[n] < 127) {
          sendMIDI(this, ShortMessage.NOTE_OFF, lastnotes[n], 0);
          lastnotes[n] = -1 ;
        }
    }
    countdownRate = defaultCountdownRate ;
  }
  String getName() {
    return "tjenn300";
  }
}
